/**
 * Add image upload dialog for Batchaction buttons.
 *
 * The `.productUiVariationattributeUnity()` method adds functionality to create a growing list of attribute values.
 *
 *
 * ### Examples
 * Apply .productUiVariationattributeUnity() to List element.
 *
 * JavaScript:
 *
 *      require([
 *        'jquery',
 *        'de_epages/product/ui/variationattribute-unity',
 *
 *        '$ready!'
 *      ], function ($, ep) {
 *        $('.ep-variationassistant-attributelist').productUiVariationattributeUnity({
 *          input: '.ep-variationassistant-attributevalueinput',
 *          initialInput: '.ep-variationassistant-attributevalueinput-initial',
 *          list: '.ep-variationassistant-attributelist',
 *          bus: $('.ep-variationassistant-attributelist').closest('form')
 *        });
 *      });
 *
 * HTML:
 *
 *    <div class="ep-variationassistant-attributelist">
 *        <div class="row ui-helper-clearfix ep-variationassistant-novaluewarning HideElement">
 *            <p>{ProvideAtLeastOneValue}</p>
 *        </div>
 *        <div>
 *            <input class="ep-variationassistant-attributevalueinput-initial ep-js" data-js=".uiValidate({setManually: 'valid', showErrorMsgAlways: true})" data-valid="true" name="VariationAttributeValues#Counter" type="text" placeholder="{Example}:&nbsp;{Blue}" >
 *            <i class="CursorPointer ep-placeInsideInput HideElement fa fa-trash fa-lg" title="{DeleteAttributeValue}"></i>
 *        </div>
 *    </div>
 *
 *    <input class="ep-variationassistant-attributevalueinput" type="text" placeholder="{Example}:&nbsp;{Red}">
 *
 *
 * @class jQuery.ui.productUiVariationattributeUnity
 *
 * @uses de_epages
 * @uses ep
 * @uses ep.dict
 * @uses ep.ui.validate
 * @uses jQuery.tmpl
 * @uses jquery.ui.widget
 * @since 7.15.0
 */

/**
 * @cfg {String} input             selector for new attribute value input
 * @cfg {String} initialInput      selector for additional initial inputs
 * @cfg {String} selectAllValues   selector for the select all toggle checkbox
 * @cfg {String} selectValue       selector for the select attribute value checkbox
 * @cfg {String} list              selector for element containing value list items
 * @cfg {String} bus               selector for element to communicate with wizard widget
 * @cfg {Integer} maxValues        maximal number of attribute values
 */

/*
 * @copyright       © Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module          de_epages/product/ui/appointmentpicker
 */


/*jslint nomen: true*/
/*global define*/

define('de_epages/product/ui/variationattribute-unity', [
    'jquery',
    'ep',
    'de_epages',
    '$tmpl!./variationattribute-unity',
    '$dict!../dictionary',

    'jquery/ui/widget',
    'jquery/metaparse',
    'ep/ui/validate'
], function ($, ep, de_epages, tmplRow, dict) {
    /*
     * @dictionary      ep.dict
     *
     * @translations    {Example}
     *                  {Blue}
     *
     * @dictionary      de_epages.product.dict
     *
     * @translations   {EmptyAttributeValue}
     *                 {DuplicateAttributeValueAlias}
     *                 {DeleteAttributeValue}
     */

    // translation object
    var translations = {
        blue: dict.translate('Blue'),
        deleteAttributeValue: dict.translate('DeleteAttributeValue'),
        doubleValue: dict.translate('DuplicateAttributeValueAlias'),
        emptyAttributeValue: dict.translate('EmptyAttributeValue'),
        example: dict.translate('Example')
    };

    $.widget('ui.productUiVariationattributeUnity', {
        timeout: {},

        options: {
            // input,
            // initialInput,
            // list,
            // bus,
            // selectValue,
            // selectAllValues,

            maxValues: 20
        },

        valid: true,
        empty: true,
        valuesSelected: false,

        _create: function() {
            var self = this,
                elem = self.element,
                o = self.options,
                $initialInput = elem.find(o.initialInput),
                showTrash = function() {
                    // display delete icon after first input and remove eventlistener
                    $initialInput.closest('.row').find('.fa-trash').removeClass('HideElement');
                    $initialInput.off('input.variationAssistent');
                };

            if (o.input === undefined || o.list === undefined) {
                return false;
            }

            self.input = elem.find(o.input);
            self.list = elem.find(o.list);
            self.bus = $(o.bus);
            self.id = elem.data('attributecount');
            self.selectAll = elem.find(o.selectAllValues);
            self.selectAllInput = self.selectAll.find('input');

            // register live event to remove rows
            self.list.on('click', '.fa-trash', self._deleteRow.bind(self));

            // register live event for error handling
            self.list.on('input', 'input[type="text"]', self._delay('validate', self._validate).bind(self));

            // register live event for validating the count of selected values
            self.list.on('change', 'input[type="checkbox"]', self._validateSelectedValueCount.bind(self));

            // register event to toggle all select value checkboxes
            self.selectAllInput.on('change', self._toggleAll.bind(self));

            // register event to insert new rows (when inserting content to input)
            self.input.on('input', self._delay('newRowInput', self._createNewRow).bind(self));



            // handle 2nd input field
            if ($initialInput.length) {
                $initialInput.one('input', function() {
                    // display delete icon after first input and remove eventlistener
                    $(this).closest('.row').find('.fa-trash').removeClass('HideElement');
                    $(this).closest('.row').find(o.selectValue).removeClass('HideElement');
                    self._validateSelectedValueCount();
                    self.selectAll.removeClass('HideElement');
                });
            }

            // if all inputs are empty, set invalid
            if (self.isEmpty()) {
                self.valid = false;
                self.bus.trigger('triggerValidation.AttributeWizard');
            }
        },

        _toggleAll: function(evt) {
            var self = this,
                o = self.options,
                target = $(evt.target);

            self.element.find(o.selectValue).prop('checked', target.prop('checked'));
            self._validateSelectedValueCount();
        },

        _deleteRow: function(evt) {
            var self = this,
                elem = self.element,
                o = self.options;

            if (self.list.find('input, .ep-variation-attribute-alias').length > 1) {
                // delete Input
                $(evt.target).trigger('change')
                    .closest('.row').remove();
            } else {
                $(evt.target).closest('.row').find('input').val('');
            }

            // revalidate
            self._validate();
            self._validateSelectedValueCount();

            // display source input if amount of rows is less than max. length of attribute values
            if (self.list.children().length < o.maxValues) {
                self.input.closest('.row').removeClass('HideElement');
            }

            if (self.isEmpty()) {
                self.selectAll.addClass('HideElement');
            }
        },

        _validateSelectedValueCount: function() {
            var self = this;

            self.bus.trigger('triggerValidation.AttributeWizard');
        },

        _validate: function(evt) {
            var self = this,
                valid = 1,
                valueElements = self.list.find('input[type="text"], .ep-variation-attribute-alias').get(),
                firstElement,
                elementValues,
                duplicates;

            // if all inputs are empty, set invalid
            if (self.isEmpty()) {
                firstElement = $(valueElements[0]);

                firstElement.addClass('DialogError');
                firstElement.uiValidate('option', {'customMessage': translations.emptyAttributeValue, 'valid': 0});
                firstElement.data('valid', false);
                firstElement.data('emptyError', true);

                self.valid = false;
                self.bus.trigger('triggerValidation.AttributeWizard');
                return;
            }

            // check if the attribute value is already used in this attribute
            // sort array so only neccessary to compare with next/previous element in array
            valueElements.sort(function(a, b) {
                var $a = $(a),
                    $b = $(b),
                    contentA = $a[$a.is('input') ? 'val' : 'text'](),
                    contentB = $b[$b.is('input') ? 'val' : 'text']();

                return contentA.localeCompare(contentB);
            });

            // filter all elements and only leave those that are same as next or previous element
            duplicates = valueElements.filter(function(element, index) {
                var $a = $(element),
                    $b = $(valueElements[index - 1]),
                    $c = $(valueElements[index + 1]),
                    contentA = $a[$a.is('input') ? 'val' : 'text'](),
                    contentC = $c[$c.is('input') ? 'val' : 'text'](),
                    contentB = $b[$b.is('input') ? 'val' : 'text']();

                return (contentA.length && (contentA.toLowerCase() === contentB.toLowerCase() || contentA.toLowerCase() === contentC.toLowerCase()));
            });

            valid = !duplicates.length;


            if (valid) {
                $(valueElements)
                    .data('valid', true)
                    .trigger('validate.uiValidate')
                    .removeClass('DialogError');

                // if validity changed, trigger global validation
                if (!self.valid && self.bus) {
                    self.bus.trigger('triggerValidation.AttributeWizard');
                }
                self.valid = true;

            } else {
                $(valueElements).removeClass('DialogError')
                    .data('valid', true)
                    .trigger('validate.uiValidate');

                duplicates = $(duplicates).filter('input');
                duplicates.addClass('DialogError');
                duplicates.each(function(index, element) {
                    $(element).uiValidate('option', {'customMessage': translations.doubleValue, 'valid': 0});
                });
                duplicates.data('valid', false);

                // if validity changed, trigger global validation
                if (self.valid && self.bus) {
                    self.bus.trigger('triggerValidation.AttributeWizard');
                }
                self.valid = false;
            }
        },

        _createNewRow: function() {
            var self = this,
                o = self.options,

                template,
                templateItems;

            // check max. length of attribute values
            if (self.list.children().length < o.maxValues) {
                // render row template with necessary values
                template = tmplRow({
                        AttributeCount: self.id,
                        tExample: translations.example,
                        tDeleteAttributeValue: translations.deleteAttributeValue,
                        tBlue: translations.blue
                    });

                templateItems = template.tmplItem('elements');

                // append template and set focus
                self.list.append(template);

                // copy content into new input and focus it
                templateItems.input.val(self.input.val()).trigger('focus');
                // move cursor to the end
                templateItems.input[0].setSelectionRange(templateItems.input.val().length, templateItems.input.val().length);
                // metaparse elements inside the template
                templateItems.input.metaparse();
                templateItems.select.metaparse();
                templateItems.select.trigger('mousedown');

                // validate after creating new row
                self._validate();
                self._validateSelectedValueCount();

                // remove content from source input
                self.input.val('');
                // hide source input if max. length of attribute values is reached
                if (self.list.children().length >= o.maxValues) {
                    self.input.closest('.row').addClass('HideElement');
                }

                self.selectAll.removeClass('HideElement');
            }
        },

        // hide list empty warning if it is visible
        removeEmptyWarning: function() {
            var self = this,
                firstElement = self.list.find('input[type="text"], .ep-variation-attribute-alias').eq(0);

                // since this is removed externally, state of error is stored;
                // only remove if this error actually exists. otherwise duplicate value error might accidentially be removed.
                if (firstElement.data('emptyError')) {
                    firstElement.removeClass('DialogError')
                        .data('valid', true)
                        .data('emptyError', false)
                        .trigger('validate.uiValidate');
                }
        },

        // returns an array containing all non-zero-length attribute values
        getSelectedValues: function(variationAttributeSelector) {
            var self = this,
                variationAttributeSelector = (variationAttributeSelector!==undefined && ep.isValidSelector(variationAttributeSelector) && ep.isJQueryElement($(variationAttributeSelector)))
                                            ? variationAttributeSelector
                                            : '.ep-variation-attribute-alias',
                values = self.list.find('input[type="text"], ' + variationAttributeSelector).filter(function(index, element) {
                    // only return the selected Values
                    return $(element).closest('.row').find('input[type="checkbox"]').prop('checked');
                }).map(function(index, element) {
                    var $element = $(element),
                        content = $element[$element.is('span') ? 'text' : 'val']();

                    return content;
                }),
                nonEmptyValues = values.filter(function(index, string) {
                    return string.length;
                });

            return nonEmptyValues.get();
        },

        // returns a boolean representing validity
        isValid: function() {
            var self = this;

            return self.valid;
        },

        // returns a boolean: true if no or only zero-length attribute values
        isEmpty: function() {
            var self = this,
                empty;

            empty = !self.list.find('input[type="text"], .ep-variation-attribute-alias').filter(function(index, element) {
                var $element = $(element),
                    content = $element[$element.is('span') ? 'text' : 'val']();

                return content.length;
            }).length;

            if (self.empty !== empty && self.bus) {
                self.bus.trigger('triggerValidation.AttributeWizard');
            }

            self.empty = empty;

            return empty;
        },

        // input delay: trigger callback only if event is not fired again within 200 ms
        _delay: function(id, cb) {
            var self = this;

            return function(evt) {
                clearTimeout(self.timeout[id]);
                self.timeout[id] = setTimeout(function() {
                    cb.bind(self)(evt);
                }, 200);
            };
        },

        destroy: function () {
            var self = this;

            self.element.html('');
            self._superApply(arguments);
        }
    });

    return de_epages;
});
